The main GCMotion namespace#

The main GCMotion namespace offers the creation of entities, such as a tokamak Profile, a Particle or a Collection of Particles, as well as tokamak configuration objects, such as a QFactor, MagneticField and ElectricField:

Constructing Quantities#

All variables that represent physical quantities (except from angles) are defind as Quantities, using the pint libary. See how you can define them here:

Tokamak Configuration#

A Tokamak entity, apart from its major and minor radii, requires a q-factor, a magnetic and an electric field to be defined:

gcmotion.qfactor

Creates a q-factor

gcmotion.bfield

Creates a Magnetic field

gcmotion.efield

Creates an Electric field

See also:

Essential Entities#

Tokamak(R, a, qfactor, bfield, efield)

Creates a tokamak device.

InitialConditions(species, theta0, zeta0, ...)

Creates a set of initial conditions for a particle

Utilities#

get_size(obj)

Recursively finds size of objects and prints the result.

Scripts#