The main GCMotion namespace#
The main GCMotion namespace offers the creation of entities, such as a tokamak Profile, a Particle or a Collection of Particles, as well as tokamak configuration objects, such as a QFactor, MagneticField and ElectricField:
Constructing Quantities#
All variables that represent physical quantities (except from angles) are defind as Quantities, using the pint libary. See how you can define them here:
Tokamak Configuration#
A Tokamak entity, apart from its major and minor radii, requires a q-factor, a magnetic and an electric field to be defined:
Creates a q-factor |
|
Creates a Magnetic field |
|
Creates an Electric field |
See also:
Essential Entities#
|
Creates a tokamak device. |
|
Creates a set of initial conditions for a particle |
Utilities#
|
Recursively finds size of objects and prints the result. |